#ifndef POLYNOMIAL_HPP
#define POLYNOMIAL_HPP

#include <cstddef> // size_t

template<typename T = float>
class ConstSizePolynomial
{
public:
    ConstSizePolynomial(size_t degree = 5);
    ConstSizePolynomial(const ConstSizePolynomial&);
    ConstSizePolynomial(ConstSizePolynomial&&);

    ConstSizePolynomial& operator=(const ConstSizePolynomial&) = delete;

    ConstSizePolynomial& operator=(ConstSizePolynomial&&);

    void setCoeff(size_t n, const T& coeff);

    T value(const T& arg) const;
    T coeff(size_t n) const;
    size_t degree() const;

    ~ConstSizePolynomial();
private:
    size_t degree_;
    T * coeffs_;
};

template<typename T = float>
class Polynomial
{
public:
    Polynomial();

    void setCoeff(size_t n, const T& coeff);

    T value(const T& arg) const;
    T coeff(size_t n) const;
    size_t degree() const;
private:
    void _resize(size_t degree);

    size_t currentPolynomialDegree_;
    ConstSizePolynomial<T> polynomial_;
};

#include "polynomial.ipp"

#endif // POLYNOMIAL_HPP
