#include <iostream>

#include "impl_3/polynomial.hpp"

#include "polynomial_printer.hpp"
#include "polynomial_derivative.hpp"
#include "polynomial_secant.hpp"
#include "polynomial_tangent.hpp"

#include "newton_method.hpp"

template<typename T = float>
auto getPolynomial()
{
    return Polynomial<T>();
}

void test_1()
{
    auto p1 = getPolynomial();

    p1.setCoeff(0, 1);
    p1.setCoeff(1, 2);
    p1.setCoeff(3, 4);

    PolynomialPrinter().print(p1);

    std::cout << "rzad wielomianu p(x) : " << p1.degree() << "\n";

    for( auto i = -2; i < 3; ++ i)
    {
        std::cout << "p(" << i << ") = " << p1.value(i) << "\n";
    }
}

void test_2()
{
    auto p1 = getPolynomial();

    p1.setCoeff(0, 3);
    p1.setCoeff(1, 2);
    p1.setCoeff(2, 1);
    p1.setCoeff(3, 4);

    auto p2 = PolynomialDerivative().derive(p1);

    std::cout << "wielomian: \n";
    PolynomialPrinter().print(p1);
    
    std::cout << "pochodna wielomianu: \n";
    PolynomialPrinter().print(p2);
    
    for( auto i = -2; i < 3; ++ i)
    {
        std::cout << "p(" << i << ") = " << p2.value(i) << "\n";
    }
}

template<typename T>
auto getSolver()
{
    // return NewtonMethod<PolynomialSecant, T>(static_cast<T>(1e-5));
    return NewtonMethod<PolynomialTangnet, T>(static_cast<T>(1e-10));
}

template<typename T>
void test_3()
{
    auto p1 = getPolynomial<T>();

    p1.setCoeff(0, -1);
    p1.setCoeff(2, 1);

    std::cout << "wielomian: \n";
    PolynomialPrinter().print(p1);

    std::cout << "wynik rozwiazania dla 10 oraz -10: \n";
    std::cout << getSolver<T>().solve(p1, static_cast<T>(10)) << "\n";
    std::cout << getSolver<T>().solve(p1, static_cast<T>(-10)) << "\n";
}

int main()
{
    std::cout << "wyklad 5 - impl 3\n";

    std::cout << "**********************************************\n";
    std::cout << "test 1 \n";
    test_1();
    std::cout << "**********************************************\n";
    std::cout << "test 2 \n";
    test_2();
    std::cout << "**********************************************\n";
    std::cout << "test 3a \n";
    test_3<float>();
    std::cout << "**********************************************\n";
    std::cout << "test 3b \n";
    test_3<double>();
    // std::cout << "**********************************************\n";
    //std::cout << "test 3c \n";
    // test_3<int>();
    
    return 0;
}
