#include "newton_method.hpp"

template<typename DerivativeMethod, typename T>
NewtonMethod<DerivativeMethod, T>::NewtonMethod(const T& eps)
: eps_(eps)
{
}

template<typename DerivativeMethod, typename T>
template<typename Polynomial>
float NewtonMethod<DerivativeMethod, T>::solve(const Polynomial& p, T arg)
{
    auto derivative_method = DerivativeMethod();

    while (p.value(arg) * p.value(arg) > eps_)
    {
        auto step = p.value(arg) / derivative_method.calculate(p, arg);
        arg = arg - step;

        if (step * step <= eps_)
        {
            break;
        }
    }

    return arg;
}
